#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>

/*
 * Functie de initializare a controllerului USART
 */
void USART0_init()
{
    /* seteaza baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 103;

    /* porneste transmitatorul */
    UCSR0B = (1<<TXEN0) | (1<<RXEN0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop, fara paritate */
    UCSR0C = (3<<UCSZ00);
}

/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART0_receive()
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

int main()
{
	USART0_init();
	char c;
	
	// configurare porturi folosite
	/* motoare */
	DDRC |= (1 << PC0);
	DDRC |= (1 << PC1);
	DDRC |= (1 << PC2);
	DDRC |= (1 << PC3);
	/* lumina */
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	/* buzzer */
	DDRB |= (1 << PB0);
	PORTB |= (1 << PB0);
	
	PORTC &= ~(1 << PC0);	// fata
	PORTC &= ~(1 << PC1);   // spate
	PORTC &= ~(1 << PC2);	// dreapta
	PORTC &= ~(1 << PC3);	// stanga
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTA &= ~(1 << PA2);

	while(1) {
		
		c = USART0_receive();
		
		switch(c) {
		case 'F':  // fata
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC1);
			break;
		case 'B':
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC0);
			break;
		case 'R':
			PORTC |= (1 << PC2);
			PORTC &= ~(1 << PC3);
			break;
		case 'L':
			PORTC |= (1 <<PC3);
			PORTC &= ~(1 << PC2);
			break;
		case 'G':	// fata-stanga
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC1);
			PORTC |= (1 <<PC3);
			PORTC &= ~(1 << PC2);
			break;
		case 'I':	// fata-dreapta
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC1);
			PORTC |= (1 << PC2);
			PORTC &= ~(1 << PC3);
			break;
		case 'H':	// spate-stanga
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC0);
			PORTC |= (1 <<PC3);
			PORTC &= ~(1 << PC2);
			break;
		case 'J':	// spate-dreapta
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC0);
			PORTC |= (1 << PC2);
			PORTC &= ~(1 << PC3);
			break;
		
		/* --- lumina si claxon --- */
		case 'W':	// aprinde lumina
			PORTA |= (1 << PA0);
			PORTA |= (1 << PA1);
			PORTA |= (1 << PA2);
			break;
		case 'w':	// stinge lumina
			PORTA &= ~(1 << PA0);
			PORTA &= ~(1 << PA1);
			PORTA &= ~(1 << PA2);
			break;
		case 'V':	// claxon on
			PORTB &= ~(1 << PB0);
			break;
		case 'v':	// claxon off
			PORTB |= (1 << PB0);
			break;
		/* stop motoare */
		default:
			PORTC &= ~(1 << PC0);
			PORTC &= ~(1 << PC1);
			PORTC &= ~(1 << PC2);
			PORTC &= ~(1 << PC3);
		}
	}
	
	return 0;
}